package org.jenkinsci.plugins.p4_client.workspace;

import hudson.Extension;
import hudson.model.AutoCompletionCandidates;
import hudson.util.FormValidation;
import hudson.util.ListBoxModel;

import java.util.List;

import org.jenkinsci.plugins.p4_client.client.ConnectionFactory;
import org.kohsuke.stapler.DataBoundConstructor;
import org.kohsuke.stapler.QueryParameter;

import com.perforce.p4java.client.IClient;
import com.perforce.p4java.client.IClientSummary;
import com.perforce.p4java.server.IOptionsServer;

public class TemplateWorkspaceImpl extends Workspace {

	private final String charset;
	private final String templateName;
	private final String format;

	public String getCharset() {
		return charset;
	}

	public String getTemplateName() {
		return templateName;
	}

	public String getFormat() {
		return format;
	}

	@Override
	public WorkspaceType getType() {
		return WorkspaceType.TEMPLATED;
	}

	@DataBoundConstructor
	public TemplateWorkspaceImpl(String charset, String templateName,
			String format) {
		this.charset = charset;
		this.templateName = templateName;
		this.format = format;
	}

	@Extension
	public static final class DescriptorImpl extends WorkspaceDescriptor {

		public static final String defaultFormat = "jenkins-${node}-${project}";

		@Override
		public String getDisplayName() {
			return "Template (view generated for each node)";
		}

		/**
		 * Provides auto-completion for workspace names. Stapler finds this
		 * method via the naming convention.
		 * 
		 * @param value
		 *            The text that the user entered.
		 */
		public AutoCompletionCandidates doAutoCompleteTemplateName(
				@QueryParameter String value) {

			AutoCompletionCandidates c = new AutoCompletionCandidates();
			try {
				IOptionsServer iserver = ConnectionFactory.getConnection();
				if (iserver != null && value.length() > 0) {
					String user = iserver.getUserName();
					List<IClientSummary> list;
					list = iserver.getClients(user, value + "*", 10);
					for (IClientSummary l : list) {
						c.add(l.getName());
					}
				}
			} catch (Exception e) {
			}

			return c;
		}

		public FormValidation doCheckTemplateName(@QueryParameter String value) {
			try {
				IOptionsServer p4 = ConnectionFactory.getConnection();
				IClient client = p4.getClient(value);
				if (client != null) {
					return FormValidation.ok();
				}
				return FormValidation.error("Unknown Client: " + value);
			} catch (Exception e) {
				return FormValidation.error("Perforce connection error.");
			}
		}

		public FormValidation doCheckFormat(@QueryParameter final String value) {
			if (value == null || value.isEmpty())
				return FormValidation
						.error("Workspace Name format is mandatory.");

			for (String l : ClientTags.getTags()) {
				if (value.contains("${" + l + "}")) {
					return FormValidation.ok();
				}
			}
			return FormValidation.error("Workspace Name format error.");
		}

	}

	/**
	 * Builds Workspace name from prefix, node and project.
	 * 
	 * @return
	 */
	@Override
	public String getName() {
		return ClientTags.getFormattedName(format);
	}
}
